<?php

namespace App\Http\Controllers;

use App\Models\Address;
use App\Models\BookableProduct;
use App\Models\Booking;
use App\Models\BookingItem;
use App\Models\TempBookingItem;
use App\Models\BookingType;
use App\Models\ContactPerson;
use App\Models\Customer;
use App\Models\Page;
use App\Models\PaymentTerm;
use App\Models\Pricelist;
use App\Services\BookingService;
use Carbon\Carbon;
use Exception;
use Illuminate\Support\Facades\Http;
use \Payson\Payments;

class PaysonController extends Controller
{
    private string $agentId;
    private string $apiKey;
    private string $apiUrl;
    private $connector;
    private $checkoutClient;

    private string $country;
    private string $currency;
    private string $locale;


    function __construct() {
        $this->bookingService = new BookingService();
        $this->agentId = config('app.payson-agent-id');
        $this->apiKey = config('app.payson-api-key');
        $this->apiUrl = Payments\Transport\Connector::TEST_BASE_URL;
        $this->connector = Payments\Transport\Connector::init($this->agentId, $this->apiKey, $this->apiUrl);
        $this->checkoutClient = new \Payson\Payments\CheckoutClient($this->connector);
        $this->currency = 'sek';

        $this->country = 'SE';
        $this->locale = 'sv-SE';
    }

    public function createOrder($id=null) {
        $bookingItems = TempBookingItem::where('uid', $id)->get()->toArray();

        $orderLines = [];
        foreach($bookingItems as $item) {
            $item_price = $item['days'] > 0 ? $item['price'] * $item['days']: $item['price'];

            $orderLines[] = [
                'name' => $item['externComments'], 
                'unitPrice' => $item_price, 
                'quantity' => $item['quantity'],
                'taxRate' => $item['tax']/100,
                'type' => 'service'
            ];
        }

        $checkoutData = [
            'order' => [
                'currency' => $this->currency,
                'items' => $orderLines
            ],
            'merchant' => [
                'checkoutUri' => url('checkout/create/'.$id),
                'confirmationUri' => url('checkout/confirm'),
                'notificationUri' => url('https://btest1.ledaco.net/demo/payson/notifications.php'),
                'termsUri' => url('terms')
            ],
            'gui' => [
                'locale' => 'sv-SE'
            ]
        ];


        $response = $this->checkoutClient->create($checkoutData);

        $data = [
            'page' => Page::where('id', 1)->first()->toArray(),
            'translations' => [
            'payment.warning' => __('payment.warning')
            ],
            'snippet' => $response['snippet'],
            'orderId' => $response['id']
        ];

        TempBookingItem::where('uid', $id)->update(['payment_provider_id' => $response['id']]);

        // Save checkout ID (token) in cookie for use on confirmation page
        setcookie('checkoutId', $response['id'], time()+60*60*24*365 , '/', ($_SERVER['HTTP_HOST'] != 'localhost') ? $_SERVER['HTTP_HOST'] : false, false);

        return view('payson-checkout', compact('data'));
    }

    public function terms()
    {
        $data = [
            'page' => Page::where('id', 1)->first()->toArray()
        ];
        return view('terms', compact('data'));
    }

    public function getOrderResponse()
    {
        $data = [
            'page' => Page::where('id', 1)->first()->toArray(),
            'snippet' => null,
            'orderId' => null,
            'error' => null
        ];

        try {
            if (isset($_COOKIE['checkoutId']) && $_COOKIE['checkoutId'] != '') {
                $id = $_COOKIE['checkoutId'];
                // Need to start check if there is actually order in database with this id, since if user reloded we do not want to show the page again
                $items = TempBookingItem::where('payment_provider_id', $id)->get();
                $stopRun = true;
                foreach($items as $ti) {
                    $stopRun = false;
                }

                if($stopRun) {
                    return redirect('/');
                }

                $pageTitle = '';
                $title = '';
                $pageDescription = '';

                $checkout = $this->checkoutClient->get(['id' => $id]);

                switch ($checkout['status']) {
                    case 'readyToShip':
                        // no need for special logic and to add to bookingItems because
                        break;
                    case 'created':
                    case 'readyToPay':
                    case 'denied':
                    case 'canceled':
                    case 'expired':
                    case 'shipped':
                    case 'paidToAccount':
                    case 'denied':
                    default:
                        throw new \Exception('Unable to show confirmation! Status: ' . $checkout['status']);
                }

                $isBookable = $this->checkIfBookingIsBookable($id);
                if($isBookable) {
                    // Need to mark the ext_bookingItem as in proccess
                    TempBookingItem::where('payment_provider_id', $id)->update(['waiting_for_payment_confirmation' => Carbon::now()]);
                }
                else {
                    $this->refundBooking($id);
                    TempBookingItem::where('payment_provider_id', $id)->delete();
                    $error = __('payment.fullybooked');
                }


                $data = [
                    'page' => Page::where('id', 1)->first()->toArray(),
                    'snippet' => !isset($error) ? $checkout['snippet'] : '',
                    'orderId' => $checkout['id'],
                    'error' => isset($error) ? $error : null
                ];
            }
            else {
                $data['error'] = 'Fel';
            }
        }
        catch(Exception $e) {
            $data['error'] = $e->getMessage();
        }
        
        return view('payson-order-confirm', compact('data'));
    }

    public function push($id=null)
    {
        // We even need this endpoint since we are getting response from Klarna directly and react right away. Should not need push to happen.
        echo $id . ' push';
    }

    public function pushCheckout($id=null)
    {
        // We even need this endpoint since we are getting response from Klarna directly and react right away. Should not need push to happen.
        echo $id . ' push';
    }

    public function pushPayment($id=null)
    {
        // We even need this endpoint since we are getting response from Klarna directly and react right away. Should not need push to happen.
        echo $id . ' push';
    }

    private function checkIfBookingIsBookable($payment_provider_id) : bool {
        $result = true;

        $bookingItems = $this->bookingService->getBookingItems($payment_provider_id);
        foreach($bookingItems as $item) {
            if($item->quantity > $item->bookableCount) {
                $result = false;
            }
        }
        // $bookingItems = TempBookingItem::where('payment_provider_id', $payment_provider_id)->get()->toArray();
        // foreach($bookingItems as $bookingItem) {
        //     $max = $this->bookingService->getProductBookableCount($bookingItem['product_id'], $bookingItem['productGroup_id'], $bookingItem['startTime'], $bookingItem['endTime'], $bookingItem['uid']);
        //     if($max < $bookingItem['quantity']) {
        //         $result = false;
        //         break;
        //     }
        //     else  {
        //         $result = true;
        //     }
        // }

        return $result;
    }

    private function refundBooking($id) {
        $response = Http::withHeaders(['Authorization' => 'Basic '.$this->authKeyBase64])
        ->post(config('app.klarna-endpoint').'ordermanagement/v1/orders/'.$id.'/cancel');
        $response = json_decode($response->getBody());
        // Need to add process here so that we get manual interaction to refund the user.
    }

    private function createBookingInBookingSystem($id, $checkout) {
        // We look up email used against customers in database
        if(isset($checkout['customer']['email'])) {
            $customer = Customer::where('email', $checkout['customer']['email'])->first();
        }

        // Get booking type external booking
        $bookingType = BookingType::where('shortName', 'ExB')->first();
        if(!isset($bookingType)) {
            $bookingType = BookingType::create([
                'name' => 'Extern Bokning',
                'shortName' => 'ExB',
                'type' => 'Extern',
                'color' => '',
                'status' => 'Aktiv'
            ]);
        }

        // Get the pricelist named Klarna
        $pricelist = Pricelist::where('priceListName', 'Externt prislista')->first();
        if(!isset($pricelist)) {
            $pricelist = Pricelist::create([
                'priceListName' => 'Externt prislista',
                'description' => 'Prislista anv&auml;nd f&ouml;r externt bokningar',
                'defaultPricelist' => 0,
                'status' => 'Aktiv'
            ]);
        }

        $paymentTerm = PaymentTerm::where('description', 'Externt')->first();
        if(!isset($paymentTerm)) {
            $paymentTerm = PaymentTerm::create([
                'description' => 'Externt',
                'bookingFee' => 0,
                'prepayment' => 0,
                'standard' => 0,
                'status' => 'Aktiv'
            ]);
        }
        
        $firstName = isset($checkout['customer']['firstName']) ? $checkout['customer']['firstName'] : '';
        $lastName = isset($checkout['customer']['lastName']) ? $checkout['customer']['lastName'] : '';
        $name = $firstName . ' ' . $lastName;

        $street = isset($checkout['customer']['street']) ? $checkout['customer']['street'] : '';
        $postalCode = isset($checkout['customer']['postalCode']) ? $checkout['customer']['postalCode'] : '';
        $email = isset($checkout['customer']['email']) ? $checkout['customer']['email'] : '';
        $phone = isset($checkout['customer']['phone']) ? $checkout['customer']['phone'] : '';
        $city = isset($checkout['customer']['city']) ? $checkout['customer']['city'] : '';

        // if we find no customer then we create one
        if(!isset($customer)) {
            $customer = Customer::create([
                'defaultPriceListId' => $pricelist->id,
                'paymentTermId' => $paymentTerm->id,
                'name' => $name,
                'contactName' => $name,
                'street' => $street,
                'street2' => '',
                'postCode' => $postalCode,
                'email' => $email,
                'emailInvoice' => '',
                'customerType' => 'Privat',
                'type' => 'Extern',
                'phone' => $phone,
                'mobile' => '',
                'social_security_number_org_nr' => '',
                'taxType' => 'Inkl',
                'city' => $city,
                'status' => 'Aktiv',
                'notes' => ''
            ]);
        }

        if(isset($customer)) {      
            $contactPerson = ContactPerson::where('email', $email)->where('customerId', $customer->id)->first();
            if(!isset($contactPerson)) {
                $contactPerson = ContactPerson::create([
                    'customerId' => $customer->id,
                    'name' => $name,
                    'email' => $email,
                    'phone' => $phone,
                    'mobile' => '',
                    'status' => 'Aktiv'
                ]);
            }

            $address = Address::where('customerId', $customer->id)->where('type', 'standard')->first();
            if(!isset($address)) {
                $address = Address::insert([
                    'customerId' => $customer->id,
                    'orgName' => '',
                    'street' => $street,
                    'street2' => '',
                    'postCode' => $postalCode,
                    'city' => $city,
                    'type' => 'standard'
                ]);
            }

            $bookingItems = TempBookingItem::where('payment_provider_id', $id)->get()->toArray();
            $start = null;
            $end = null;
            $project = null;
            foreach($bookingItems as $item) {
                $tmp_start = Carbon::createFromFormat('Y-m-d H:i:s', $item['startTime']);
                $tmp_end = Carbon::createFromFormat('Y-m-d H:i:s', $item['endTime']);
                if(!isset($project)) {
                    $project = $item['project'];
                }

                if(!isset($start) && isset($tmp_start)) {
                    $start = $tmp_start;
                }
                else if(isset($start) && $tmp_start->lt($start)) {
                    $start = $tmp_start;
                }

                if(!isset($end) && isset($tmp_end)) {
                    $end = $tmp_end;
                }
                else if(isset($end) && $tmp_end->gt($end)) {
                    $end = $tmp_end;
                }
            }

            $booking = Booking::create([
                'customerId' => $customer->id,
                'priceListId' => $pricelist->id,
                'contactPersonId' => $contactPerson->id,
                'bookingTypeId' => $bookingType->id,
                'paymentTermId' => $paymentTerm->id,
                'agreement' => '',
                'project' => $project,
                'preacher' => '',
                'startTime' => $start->format('Y-m-d H:i:s'),
                'endTime' => $end->format('Y-m-d H:i:s'),
                'bookingType' => 'extern bokning',
                'type' => 'Extern',
                'description' => 'Extern bokning',
                'guestQuantity' => 0,
                'ventilation' => '',
                'status' => 'Betald',
                'notes' => '',
                'notes2' => '',
                'ourReference' => 'Extern bokning',
                'invoiceReference' => '',
                'discountPercent' => 0,
                'publicNotes' => '',
                'offering' => '',
                'meetingOrganizer' => '',
                'music' => '',
                'canticle' => '',
                'technology' => '',
                'take' => '',
                'team' => '',
                'producer' => '',
                'endedTextId' => 0,
                'pricePerPerson' => 0,
                'publish' => 0,
                'bookingCopyId' => 0,
                'createdBy' => 'Extern',
                'createdDate' => Carbon::now(),
                'lastEditedBy' => 'Extern',
                'lastEditedDate' => Carbon::now()
            ]);

            foreach($bookingItems as $item) {
                $bookingItem = BookingItem::create([
                    'bookingId' => $booking->id,
                    'productId' => $item['product_id'],
                    'comments' => '',
                    'externComments' => $item['externComments'],
                    'tax' => $item['tax'],
                    'price' => $item['price'],
                    'quantity' => $item['quantity'],
                    'days' => $item['days'],
                    'provisional' => '0',
                    'responsibleKitchen' => '',
                    'responsibleCatering' => '',
                    'quantityType' => '',
                    'startTime' => $item['startTime'],
                    'endTime' => $item['endTime'],
                    'discountValue' => '0.00',
                    'discountType' => 'percent',
                    'movable' => '1',
                    'altText' => '',
                    'account' => '',
                    'resultUnit' => '',
                    'project' => $item['project'],
                    'parentId' => 0,
                    'eventUid' => $item['eventUid'],
                    'ext_paymentItem_id' => $item['ext_paymentItem_id']
                ]);

                if(isset($bookingItem)) {
                    TempBookingItem::where('id', $item['id'])->delete();
                }
            }
        }
    }
}